# Built-in imports
from platform import system, machine

# Package imports
from selenium import webdriver
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.chrome.options import Options
from webdriver_manager.chrome import ChromeDriverManager

# Setup Driver
def setup_driver(headless=False, is_linux_armv7l=(system() == "Linux" and machine() == "armv7l")) -> webdriver.Chrome:
    options = Options()

    if (headless):
        headless_opts = [
            "--headless",
            "--disable-gpu",
            "--window-size=1920,1200",
            "---ignore-certificate-errors",
            "--disable-extensions",
            #"--no-sandbox",
            #"--disable-dev-shm-usage",
            #"--remote-debugging-port=9515",
            #"--disable-setuid-sandbox"
        ]
        for opt in headless_opts:
            options.add_argument(opt)
            #print(opt)
    
    # Much thanks to https://stackoverflow.com/a/71042821
    try:
        if not is_linux_armv7l:
            service = Service(ChromeDriverManager().install())
        else:
            #display = Display(visible=0, size=(1920,1200))
            #display.start()
            raspbian_chromium = "/usr/lib/chromium-browser/chromedriver"
            service = Service(raspbian_chromium)
            #options.binary_location = raspbian_chromium
    except:
        # Attempt selenium fallback
        service = Service()
                

    return webdriver.Chrome(service=service, options=options)





# Built-in imports
from os import getenv, listdir, remove
from os.path import realpath, dirname, join, isfile
from pathlib import Path
from sys import argv
from json import load
from datetime import datetime

# Package imports
from dateutil import parser
from jinja2 import FileSystemLoader, Environment, PrefixLoader, PackageLoader
from slugify import slugify

# Local imports
from ..Event import load_events_from_json


try:
    loader = PackageLoader("htmlexport")
except:
    template_dir = join(realpath(dirname(__file__)), "templates")
    loader = FileSystemLoader(template_dir)

jinja_env = Environment(
    loader=loader,
    autoescape=True,
    trim_blocks=True,  # Thanks to https://stackoverflow.com/a/35777386
    lstrip_blocks=True,
)

jinja_env.filters["slug"] = slugify


# img_dir: absolute path to look for in images
# img_path_relative_to_index: relative path from index.html to where images are stored on the host
# See Event.py
def events_to_html(
        events: list, 
        absolute_img_dir: str, 
        relative_from_html_img_dir: str,
        page_urls: list, 
        output_dir: str,
        title="Document Title",
        host_domain="https://domain.example.com",
        timezone="Europe/London"):
    """
    events: Events to export
    page_urls: 
    """
    template_index = jinja_env.get_template("index.html")

    sources = set([event.source for event in events])
    
    output = template_index.render(
        # 
        events=events,
        # Absolute image
        image_dir=absolute_img_dir,
        # Image path to be used in html to load images
        img_path_relative_to_index=relative_from_html_img_dir,
        title=title,
        domain=host_domain,
        timezone=timezone,
        pages=page_urls,
        sources=sources,
        now=datetime.now())

    filename_index = join(output_dir, "index.html")
    with open(filename_index, "w") as file:
        file.writelines(output)
    
    return output



# if __name__ == "__main__":
#     events_json = realpath(argv[1])
#     dir = dirname(events_json)

#     events = load_events_from_json(events_json)

#     events_to_html(events, dir)


from time import sleep

from .driver import setup_driver
from ..utils.url_converter import facebook_www_to_locale

from selenium.webdriver.common.by import By

from os import getenv
from os.path import join, realpath
from time import sleep
from json import loads
from urllib.request import urlretrieve

# Package imports
from selenium.webdriver.common.by import By
from selenium.common.exceptions import NoSuchElementException

# Local imports
from ..utils.fb_regexes import find_and_remove, regex_in, re_line_with_characters, re_guests, re_three_letter_two_digit_date, re_utc_time, re_utc_and_more
from ..Event import Event
from .driver import setup_driver
from ..utils.url_converter import facebook_www_to_locale

from .utils import save_image
from .scrape_event_page import scrape_event_page

from time import sleep

def scrape_events_page(driver, event_url, img_dir) -> list[Event]:
    driver.get(facebook_www_to_locale(event_url))
    sleep(20)
    source = driver.find_element(By.XPATH, "//h1").text.strip()
    event_container = driver.find_element(By.XPATH, """//div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div/div/div[4]/div/div/div/div/div/div/div/div/div[3]""")
    raw_events = event_container.find_elements(By.XPATH, "*")
    events = []
    for event in raw_events:
        print("Detected upcoming event in {}!".format("page"))
        raw_data = event.text
        if regex_in(raw_data, re_utc_and_more): # Events with multiple times have multiple entries + one main. This is the main, so skip it
            print("detected main event with times, skipping current")
            continue

        try:
            lines = raw_data.split("\n")
            name = lines[1]
            datetime = lines[0]
            url = ""
            location = ""
        except IndexError:
            print("Failed to add event from page")
            print("Provided data: {}".format(lines))
            continue
    
        try:
            # find_element doesn't work, perhaps due to grandchild?
            urls = event.find_elements(By.TAG_NAME, "a")
            url = urls[0].get_attribute("href")
        except IndexError:
            print("No url found")
        
        location, image_url = scrape_event_page(url)

        event = Event(name, datetime, source, location, url)
        events.append(event)

        if image_url:
            save_image(event, image_url, img_dir)

        
    return events


from os import getenv, listdir, remove
from os.path import realpath, dirname, join, isfile
from pathlib import Path

# This will remove any images in the provided directory that aren't from provided events
def cleanup_images(still_upcoming_events: list, directory: str):
    upcoming_events_slugs = [event.uid for event in still_upcoming_events]

    files = listdir(directory)
    for filename in files:
        file = join(directory, filename)
        if isfile(file):
            filename_no_ext_or_path = Path(file).stem
            if filename_no_ext_or_path not in upcoming_events_slugs:
                remove(file)



from os.path import join
from urllib.request import urlretrieve 

def read_pages_from_env(replace_locale=True):
    """ LOADING & PARSING PAGES FROM .ENV """
    raw_pages = loads(getenv("pages"))
    pages = []
    for raw_page in raw_pages:
        page_type = raw_page[0].lower().strip()
        url = raw_page[1]

        match page_type:
            case "page":
                func = parse_page
            case "community":
                func = parse_community
            case _:  # Default
                func = parse_page
    
        if replace_locale:
            # Specify localisation
            url = facebook_www_to_locale(url)

        pages.append((func, url))
    return pages


def save_image(event, image_url, img_dir):
    if ".png" in image_url:
        ext = ".png"
    else:
        ext = ".jpg"
    print(event.uid)
    urlretrieve(image_url, join(img_dir, event.uid + ext))



# Built-in imports
import re

""" Regexes to parse input from Facebook """

"""
Example:
    NOV
    11
    Text

    Returns: 
        Nov
        11
"""
re_three_letter_two_digit_date = r"^[a-zA-Z]{3}\W\d{1,2}$"

"""
Example:
    Fri 19:00 UTC+01 · 84 guests

    returns: Fri 19:00 UTC+01
"""
re_utc_time = r".*UTC\+\d{2}"

"""
Example:
    NOV
    19
    Name - 32 guests

    Returns: Name - 32 guests 
"""
re_guests = r".*guest.*"

# Matches any line with 1 or more characters
re_line_with_characters = r"^.{1,}$"

"""
Example:
    WED, 5 NOV AT 08:30 UTC+01 AND 1 MORE

    Returns: UTC+01 AND 1 MORE

"""
re_utc_and_more = r"UTC\+\d* AND \d{1,} MORE"

def regex_in(data, pattern):
    return bool(re.search(pattern, data))

def find_and_remove(data, pattern):
    try:
        found = re.search(pattern, data, flags=re.MULTILINE).group()
        data = data.replace(found, "")
    except AttributeError as e:
        print("Error during {0} Regex search".format(pattern))
        found = ""
    finally:
        return data, found
